# ../scripts/custom/gg_give/gg_give.py

'''
    (c) 2010 Clanbake (Chris Konieczny http://www.chriskonieczny.com)
    Title: gg_give
    Version: 1.0.2 for GunGame 5.1
    Description:
    Fast (flexible) reload for gg5.1. Registers both say and client commands,
    configurable per round, per map, give delay and more...
'''

# ==============================================================================
#   IMPORTS
# ==============================================================================
# Python imports
import time

# EventScripts imports
import es
import gamethread
from cmdlib import registerSayCommand
from cmdlib import unregisterSayCommand
from cmdlib import registerClientCommand
from cmdlib import unregisterClientCommand
from playerlib import getPlayer

# GunGame Imports
from gungame51.core.addons.shortcuts import AddonInfo
from gungame51.core.addons import PriorityAddon
from gungame51.core.players.shortcuts import Player
from gungame51.core.messaging.shortcuts import msg
from gungame51.core.messaging.shortcuts import hudhint

# ==============================================================================
#   ADDON REGISTRATION
# ==============================================================================
# Register this addon with EventScripts
info = AddonInfo()
info.name     = 'gg_give'
info.title    = 'GG Give'
info.version  = '1.0.2'
info.author   = 'Clanbake (Chris Konieczny http://www.chriskonieczny.com)'
info.translations = ["gg_give"]

# ==============================================================================
#   GLOBALS
# ==============================================================================
# global variables
# User defined
giveCommand = str(es.ServerVar("gg_give_command"))
delayUsage = int(es.ServerVar("gg_give_delay_usage"))
limitPerRound = int(es.ServerVar("gg_give_per_round"))
limitPerMap = int(es.ServerVar("gg_give_per_map"))
weaponExceptions = str(es.ServerVar("gg_give_weapon_exceptions"))
showStatusHud = int(es.ServerVar("gg_give_show_stats"))
warmupGive = int(es.ServerVar("gg_give_warmup_round"))
dispHudOnly = int(es.ServerVar("gg_give_disp_msgs_in_hud"))
# End user defined

if weaponExceptions:
    weaponExceptionsList = weaponExceptions.split(',')
else:
    weaponExceptionsList = 0

if limitPerRound >= limitPerMap and limitPerMap != 0:
    limitPerRound = 0

giveDisabled = False
weaponMapLimitReached = False
weaponRoundLimitReached = False
dictUsedPerMap = {}
dictUsedPerRound = {}
dictUsedLast = {}

# ==============================================================================
#   GAME EVENTS
# ==============================================================================
def load():           
    es.dbgmsg(0, "Loaded: %s" % info.name)

    # Register the say and client command
    registerSayCommand(giveCommand, giveGun, "Give gun via say command.")
    registerClientCommand(giveCommand, giveGun, "Give gun via client command.")

def unload():
    es.dbgmsg(0, "Unloaded: %s" % info.name)    

    # Un-register say or client command
    unregisterSayCommand(giveCommand)
    unregisterClientCommand(giveCommand)

def round_start(ev):
    global weaponRoundLimitReached
    dictUsedPerRound.clear()
    dictUsedLast.clear()
    weaponRoundLimitReached = False    

def es_map_start(ev):
    global weaponRoundLimitReached
    global weaponMapLimitReached
    dictUsedPerRound.clear()
    dictUsedPerMap.clear()
    dictUsedLast.clear()
    weaponRoundLimitReached = False
    weaponMapLimitReached = False
    
# ==============================================================================
#   MAIN FUNCTION
# ==============================================================================      
def giveGun(userid, args):
    global weaponRoundLimitReached, weaponMapLimitReached
    
    if warmupGive == 0:
        if PriorityAddon():
                showSayMsg(userid, 'NotAllowedWarmup', {'command': giveCommand})
                return
 
    if getPlayer(userid).isdead or es.getplayerteam(userid) < 2:
        return
        
    ggPlayer = Player(userid) 
    weapon = ggPlayer.weapon

    if weaponRoundLimitReached == False and weaponMapLimitReached == False:  
        if weapon in weaponExceptionsList:
            if dispHudOnly:
                showHudMsg(userid, 'NotAllowedHud', {'command': giveCommand, 'weapon': weapon})
            else:
                showSayMsg(userid, 'NotAllowed', {'command': giveCommand, 'weapon': weapon})
            
            return
 
        if delayUsage:
            if not giveGunTimer(userid):
                if dispHudOnly:
                    showHudMsg(userid, 'UsageDelayHud', {'command': giveCommand, 'delay': delayUsage})   
                else:
                    showSayMsg(userid, 'UsageDelay', {'command': giveCommand, 'delay': delayUsage})
                    
                return     
      
    if limitPerRound:
        if not getGiveLimit(userid,dictUsedPerRound,limitPerRound):
            giveStats(userid)
            weaponRoundLimitReached = True
            return
 
    if limitPerMap: 
        if not getGiveLimit(userid,dictUsedPerMap,limitPerMap):
            giveStats(userid)
            weaponMapLimitReached = True
            return      
        
    giveStats(userid) 

    if getPlayer(userid).isdead or es.getplayerteam(userid) < 2:
	return

    ggPlayer.strip(True)
    gamethread.delayed(0.05, ggPlayer.give_weapon)


# ==============================================================================
#   HELPER FUNCTIONS
# ============================================================================== 
def getGiveLimit(userid,dictUsage,limitForType):
    if not userid in dictUsage:
        dictUsage[userid] = 1
        return 1
    if limitForType <= dictUsage[userid]:
        return 0    
    dictUsage[userid] += 1
    return 1

#TODO: fix - bit of kludge here  
def giveStats(userid):
    if showStatusHud:
        if limitPerRound or limitPerMap:
            round = -1
            map = -1
            if limitPerRound:
                if userid in dictUsedPerRound:
                    round = limitPerRound - dictUsedPerRound[userid]
                else:
                    round = limitPerRound
                
            if limitPerMap:  
                if userid in dictUsedPerMap:
                    map = limitPerMap - dictUsedPerMap[userid]
                else:
                    map = limitPerMap
  
            if round != -1 and map != -1 and round >= map:
                round = map
    
            if round >= 0:
                round = 'Round: %s / %i' % (round, limitPerRound)
            else:
                round = ''
            
            if map >= 0:
                map = 'Map: %s / %i' % (map, limitPerMap)
            else:
                map = ''

            if dispHudOnly:
                showHudMsg(userid, 'HudStats', {'stats': round + '\n' + map})
            else:
                showSayMsg(userid, 'SayStats', {'stats': round + ' - ' + map})

def showHudMsg(userid, string, tokens):
    hudhint(userid, string, tokens) 

def showSayMsg(userid, string, tokens):
    msg(userid, string, tokens)

def giveGunTimer(userid):
    if not userid in dictUsedLast:
        dictUsedLast[userid] = time.time()
        return 1
    if time.time() - dictUsedLast[userid] > delayUsage:
        dictUsedLast[userid] = time.time()
        return 1
    return 0
